##Chapter 22
In this hour, we mentioned Postman as a way to interact with an API.  You can certainly still use Postman as well; however, we also briefly mentioned Swagger.  The API project included here (Chapter-22-API) includes Swagger and Swashbuckle, a package that sets up a UI for Swagger.  Run the project and navigate to /swagger in a browser to find the Swashbuckle UI giving you access to test all of the endpoints in your Web API.  Because Swagger only looks at your API, it doesn't know about other endpoints up the chain in the request pipeline: namely, the /token endpoint for OAuth bearer authentication.  You should use Postman to test that endpoint and you can use Swagger to test the rest.  Bonus points if you figure out how to pull your Swagger file into Postman directly.

If you like, deploy the API to Microsoft Azure and interact with it from there.  Use the Microsoft Azure deployment guidelines from Hour 17, "Demonstrating Deployment" to get started.  You'll need Visual Studio (VS Express is free) and you'll want to activate your free Azure credit so this doesn't cost you anything.  Also, you'll want to setup a SQL Azure database and set the ```DefaultConnection``` connection string on your API accordingly.

Now that you have the API running locally and/or on Azure and you've tested it with Swagger, let's build out account registration and login!  Create a register view that takes user input for the fields present in the API's ```RegisterBindingModel``` class.  Create a ```register.controller.js``` to coordinate input from the view to an ```authentication.factory.js``` (create that too).  For login, create a view that captures email and password, a ```login.controller.js```, and a login function on ```authentication.factory.js```.  For the login function, manually set the ```Content-Type``` header to ```application/x-www-form-urlencoded``` and include a ```grant_type``` parameter with a ```"password"``` value in the request payload.  Run the app and test that you can register for an account against your API and obtain a token by logging in as well.

Before we start pulling data from our API, we have to configure our application to pass our authentication token along with each request.  This will allow the server to identify the user associated with the request and just return back that user's list of books from the database.  We can use an _interceptor_ to accomplish this.  Interceptors run for every HTTP request so they're great for things like authentication global success/error handling.  Create a file called ```api-interceptor.js``` and in its ```.config()``` call, push your interceptor function onto the ```$httpProvider```'s interceptors array.  This interceptor can now set the Authorization header to the token from the API.

Our interceptor also depends on another factory called ```LocalStorageManager``` which manages reading and writing from local storage.  Your login function in ```authentication.factory.js``` should write the token to local storage using this manager (hint: inject it as a dependency).  You may include "logout" functionality that clears your token from local storage. Back on the interceptor side, we'll want to read the token out of local storage and send it along to any request to our API.  Be careful to exclude this token from other external APIs (such as Google Books): their server may not fulfill your request correctly otherwise.  The interceptor must return an object with three functions: ```request```, ```response```, ```responseError```.  The first will run on each request.  This is where we want to add our conditional Authorization header logic.  The second runs on each successful response.  You could show success messages globally here if you like.  The third runs on each API response error.  Use this to do global error handling for you app.  Bonus points if you pull in a UI library like Toastr to show red error messages when errors occur.

Finally, let's start using the API!  In ```book.controller.js``` add a ```getBooks()``` method and call it in ```activate()``` so it runs on page load.  This should call a ```getBooks()``` method on the book factory.  The factory method should use the API factory's ```Get()``` method to fetch the list of books.  Similarly, we'll want to add a ```postBook()``` method to the book factory and use the API factory's ```Post()``` method to create a book.

We injected an Angular Value service called ```appConstants``` that maintains static config data like the location of our API.  You can set that up in ```app.js```.
Use the API prefix in your book factory when building the ```getBooks()``` and ```postBook()``` functions.

Run your app and it should load your list of books (which may be none on your first time).  Add a book and refresh the page.  Woo data persistence!  Next steps for you include edit and delete functionality for your book list.

N.B. At the time of writing, ```.success()``` and ```.error()``` were supported promise methods for HTTP calls.  As of Angular 1.4, these are deprecated in favor of using the standard ```.then()``` with two functions as parameters: success and error.  If you run into issues with this, replace these methods with ```.then()```.