(function(){
    angular.module('book')
        .config(['$httpProvider', function ($httpProvider) {
            $httpProvider.interceptors.push('APIInterceptor');
        }])
        .service('APIInterceptor', APIInterceptor);

    APIInterceptor.$inject = ['$q', 'appConstants', 'LocalStorageManager'];

    function APIInterceptor($q, appConstants, LocalStorageManager) {

        var service = this;
        service.request = handleRequest;
        service.response = handleResponse;
        service.responseError = handleResponseErrors;

        function handleRequest(config){
            if(config.url.indexOf(appConstants.apiPrefix) !== -1){
                config.headers.Authorization = getToken();
            }
            return config;
        }

        function handleResponse(response){
            return response;
        }

        function handleResponseErrors(response) {
            return $q.reject(response);
        }


        function getToken() {
            if (LocalStorageManager.DoesExist(appConstants.tokenKey)) {
                var tokenObject = LocalStorageManager.Get(appConstants.tokenKey);
                var now = new Date();
                var expirationDate = new Date(tokenObject['.expires']);
                if (now > expirationDate) {
                    LocalStorageManager.Remove(appConstants.tokenKey);
                } else {
                    var tokenHeader = tokenObject.token_type + ' ' + tokenObject.access_token;
                    return tokenHeader;
                }
            }
            return null;
        }


    }
})();