(function(){
	angular.module('book')
		.factory('bookFactory', bookFactory);

	bookFactory.$inject = ['api', 'appConstants'];

	function bookFactory(api, appConstants){
		return {
			getBooks: getBooks,
			addBook: addBook,
			deleteBook: deleteBook,
			updateBook: updateBook
		};

		function getBooks() {
			return api.Get(appConstants.apiPrefix + '/api/book');
		}

		function addBook(book){
			return getSearchResults(book)
				.then(function(response) {

					var price = (response.data.items[0].saleInfo.listPrice)
						? response.data.items[0].saleInfo.listPrice.amount
						: 0;

					var currency = (response.data.items[0].saleInfo.listPrice)
						? response.data.items[0].saleInfo.listPrice.currencyCode
						: 'USD';

					var newBook = {
						title: response.data.items[0].volumeInfo.title,
						author: response.data.items[0].volumeInfo.authors[0],
						description: response.data.items[0].volumeInfo.description,
						publicationDate: response.data.items[0].volumeInfo.publishedDate,
						publisher: response.data.items[0].volumeInfo.publisher,
						numberOfPages: response.data.items[0].volumeInfo.pageCount,
						averageRating: response.data.items[0].volumeInfo.averageRating,
						isbn: response.data.items[0].volumeInfo.industryIdentifiers[0].identifier,
						price: price,
						currency: currency
					};

					return postBook(newBook).success(function(response){
						return response;
					}).error(function(error) {
						return error;
					});

				}
			);

		}

		function deleteBook() {
			//You should implement this later
		}

		function updateBook() {
			//You should implement this later
		}

		function getSearchResults(book){
			var queryString = book.title.replace(" ", "+") + '+inauthor:' + book.author.replace(" ", "+");

			return api.Get('https://www.googleapis.com/books/v1/volumes?q=' + queryString);
		}


		function postBook(book) {
			return api.Post(appConstants.apiPrefix + '/api/book', book);
		}


	}
})();
